#!/usr/bin/rexx
/*
   Name:    setupBSF.rex
   Purpose: set up BSF4Rexx such, that it can be used easily (attempt to serve users
            without any knowledge about Java installation issues)
   Author:  Rony G. Flatscher
   Date:    2006-01-07
   Version: 1.08
   Changes:
            2006-01-07, ---rgf, incorporated Mark Miesfeld fix (wording, parseArgs)
                                fixed bug, if "." was missing in set CLASSPATH
            2006-01-15, ---rgf, on Windows make sure that path chunks get quotes removed
            2006-01-17, ---rgf, split jar-file into bsf-v???-yyyymmdd.jar and bsf-rexx-engine.jar,
                                take that into account for copying and removing
            2006-01-22, ---rgf, on Linux hint the user how to enter "setEnvironment4BSF4Rexx.sh"
            2006-01-24, ---rgf, make sure that support infos are quoted with apostrophes on Unix,
                                with quotes on Windows, added qc() for that purpose
            2006-02-06, ---rgf, Unix version now prepared to add setEnvironment4BSF4Rexx.sh to .bashrc
                                automatically
            2006-02-25, ---rgf, "setEnvironment4BSF4Rexx.*" is now copied to a PATH directory,
                                corrected quoting for Unix echo-outputs
            2006-08-01, ---rgf, fixed a bug which prevented from "setEnvironment4BSF4Rexx.*" to be
                                deleted upon uninstall; corrected grammar error; takes care to
                                copy "commons-logging-1.1.jar", if necessary; fixed a bug, now
                                creates working "bsf4rexx.{cmd|sh}"

   last change: $Revision: 170 $ $Author: rony $ $Date: 2006-08-02 00:11:28 +0200 (Wed, 02 Aug 2006) $

   Usage:   rexx setupBSF [path2java.exe [, path2binDir-to-install-Rexx-scripts-to]]
            ... "path2java.exe": optional path to java executable
            ... "path2binDir-to-install-Rexx-scripts-to": optional path to directory
                into which the supporting rexx-scripts should be copied to; defaults
                to "/usr/bin" on Linux and to the ooRexx-homedirectory on Windows

   license:

    ------------------------ Apache Version 2.0 license -------------------------
       Copyright (C) 2006 Rony G. Flatscher

       Licensed under the Apache License, Version 2.0 (the "License");
       you may not use this file except in compliance with the License.
       You may obtain a copy of the License at

           http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing, software
       distributed under the License is distributed on an "AS IS" BASIS,
       WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
       See the License for the specific language governing permissions and
       limitations under the License.
    -----------------------------------------------------------------------------
*/

cmd.=""                          /* default value for stem        */
cmd.eArgSeparator="09"x          /* separator for delimiting Rexx arguments   */
                                 /* paths to Java and desired bin-dir given?  */
parse value parseArgs(arg(1)) with cmd.eJavaBinary (cmd.eArgSeparator) cmd.eBinDir

cmd.=""                          /* default value for stem        */
call getJavaProps                /* get the properties of the installed Java  */
/*
call dumpJavaProps               /* dump the Java properties in sorted order  */
*/

/* for forcing testing Unix on Windows
j.file.separator="/" -- rgf, debug
j.path.separator=":" -- rgf, debug
*/

   /* parse path, put directories into the stem variable */
path.=""
call parsePath

if j.path.separator="" | j.file.separator="" then
do
   say "PANIC: could not use 'java' to get at its properties!"
   say
   say "Please make sure that you have Java installed! You can run 'setupBSF.rex'"
   say "with an argument being the fully qualified path to the 'java' executable"
   say "for which you wish to create the 'un/install' and 'bsf4rexx' scripts."
   say
   call usage
end

      /* determine commands depending on Unix or Windows (OS2)    */
cmd.eIsUnix=(j.file.separator="/")
      /* DLL/SO-extension library usually named "i386" on Intel, and "solaris" for Sun's Java   */
cmd.eExt.dir.bin=j.java.home || j.file.separator || "lib" || j.file.separator || ,
                                iif(pos("86", j.os.arch)>0, "i386", j.os.arch)



      /* determine CLASSPATH environment variable value  */
call SysFileTree "bsf-*.jar", "cmd.eJarBaseNames.", "FO" /* query for the jars we need */
cmd.eSetCP=iif(cmd.eIsUnix=1, "", "set ") || "CLASSPATH="
do i=1 to cmd.eJarBaseNames.0    /* iterate over all found jar-files    */
   cmd.eSetCP=cmd.eSetCP || iif(i=1, "", j.path.separator) || cmd.eJarBaseNames.i
end

/* 2006-08-02, ---rgf */
     /* get and set the commons-logging jar-file name: */
   cmd.eCommonsLoggingJarName=""    /* default to empty  */
   call sysFileTree "commons-logging-*.jar", "clname.", "FO"   /* get fully qualified file name */
   if clname.0>0 then
   do
      cmd.eCommonsLoggingJarName=clname.1    /* save very first (should be only) filename */

         /* extend classpath  */
      cmd.eSetCP=cmd.eSetCP || j.path.separator || cmd.eCommonsLoggingJarName

         /* add to jar-stem   */
      cmd.eJarBaseNames.0=i         /* i got already increased by 1, so no need to do it again */
      cmd.eJarBaseNames.i=cmd.eCommonsLoggingJarName
   end
/* 2006-08-02, ---rgf */

cmd.eSetCP=cmd.eSetCP || j.path.separator || "." || j.path.separator || -
                         iif(cmd.eIsUnix=1, "$CLASSPATH" "; export CLASSPATH", "%CLASSPATH%")


   /* define path to "jvm.dll/libjvm.so" ("javai.dll/libjavai.so" for Java 1.1.8)   */
cmd.ePath2JavaDLL= getPath2JavaDll()

cmd.eSupportInfos=qc("Support: Internet newsgroup <news:comp.lang.rexx> or via <http://www.RexxLA.org>")



if cmd.eIsUnix then
do
   nl="0a"x                      /* Unix new-line character sequence (\n)     */
   cmd.eLeadIn="#!/bin/sh"nl     /* command to execute this file              */
   cmd.eEcho="echo"
   cmd.eCopy="cp -plfv"          /* preserve, hard-link, force, verbose: source target(dir)  */
   cmd.eLink="ln -fv"            /* force, verbose: source target(dir)  */
   cmd.eDelete="rm -fv"
   cmd.eSetPath="PATH="               ||     j.user.dir    || j.path.separator || "$PATH" "; export PATH"
   cmd.eInstallScriptName="installBSF4Rexx.sh"
   cmd.eUninstallScriptName="uninstallBSF4Rexx.sh"
   cmd.eTestScriptName   = "bsf4rexx.sh"  /* "testBSF4Rexx.sh"             */
   cmd.eSetEnvironmentScriptName="setEnvironment4BSF4Rexx.sh"

   cmd.eAdd2BashRc   = "~/.bashrc"   /* bash profile to add entry to        */
   cmd.eAdd2BashRc.0 = 3
   cmd.eAdd2BashRc.1 = "    # allow BSF4Rexx to find the Java dynamic library (such that Java can be demand loaded)"
   cmd.eAdd2BashRc.2 = "    ." cmd.eSetEnvironmentScriptName ">/dev/null"

   cmd.eSetExecutable    ="chmod 755"     /* set executable for all (user, group, other)  */
   cmd.eLineComment="#"
   cmd.eDLL="libBSF4Rexx.so"     /* define name of dynamic link library    */
   if cmd.eBinDir="" then
      cmd.eBinDir="/usr/bin"     /* directory where executables reside     */

   cmd.eLibDir="/usr/lib"
   cmd.eQuote=''                 /* no quoting of file names for Unix      */
   cmd.eAllArguments='$*'
   if cmd.eJavaBinary="" then
      cmd.eJavaBinary=getFromPath("java")    /* get fully qualified path   */
end
else  /* Windows  */
do
   nl="0d0a"x                    /* Windows/OS2 new-line character sequence (\n) */
   cmd.eSetLocal="setlocal"      /* Windows command to set a local environment*/
   cmd.eEcho="@echo"
   cmd.eEchoOff="@echo off"
   cmd.eCopy="copy /y"
   cmd.eLink=cmd.eCopy           /* no ln available, hence copy   */
   cmd.eDelete="del /f"
   cmd.eSetPath="set PATH="      || j.user.dir        || j.path.separator || "%PATH%"
   cmd.eSetDLLPath="set PATH="   || cmd.ePath2JavaDLL || j.path.separator || "%PATH%"
   cmd.eInstallScriptName="installBSF4Rexx.cmd"
   cmd.eUninstallScriptName="uninstallBSF4Rexx.cmd"
   cmd.eTestScriptName   = "bsf4rexx.cmd" /* "testBSF4Rexx.cmd"            */
   cmd.eSetEnvironmentScriptName="setEnvironment4BSF4Rexx.cmd"
   cmd.eLineComment="REM "
   cmd.eDLL="BSF4Rexx.dll"       /* define name of dynamic link library    */
   if cmd.eBinDir="" then
      cmd.eBinDir=getRexxDir()   /* directory where executables reside     */

   cmd.eLibDir=cmd.eBinDir
   cmd.eQuote='"'                /* always quote file names for Windows    */
   cmd.eAllArguments='%*'
   if cmd.eJavaBinary="" then
      cmd.eJavaBinary=qw(getFromPath("java.exe"))   /* get fully qualified path   */
end

if cmd.eJavaBinary="" | cmd.eJavaBinary='""' then
do
   javaName="'java"iif(cmd.eIsUnix,"",".exe")"'"
   say "PANIC: no executable" javaName "found !"
   say
   say "Please make sure that you have Java installed! You can run 'setupBSF.rex'"
   say "with an argument being the fully qualified path to the" javaName "executable"
   say "for which you wish to create the 'un/install' and 'bsf4rexx' scripts."
   say
   call usage
end

tmpStr=cmd.eJavaBinary "-Djava.library.path=" || cmd.eQuote || j.user.dir /* make sure Java finds BSF4Rexx-dll   */

if j.java.library.path<>"" then                 /* add defined paths as well           */
do
   if cmd.eIsUnix=0 then
      tmpStr=tmpStr || j.path.separator || j.java.library.path
   else     /* Windows? Make sure no quotes within path  */
      tmpStr=tmpStr || j.path.separator || changestr('"', j.java.library.path, '')
end

tmpStr=tmpStr || cmd.eQuote

cmd.eTestStartRexx=tmpStr "org.rexxla.bsf.RexxDispatcher" cmd.eAllArguments

   /* create the scripts   */
call createTestScript

call createEnvironmentScript  /* Windows probably needs this script        */

call createInstallScript   /* also creates the matching uninstall script      */

call info                  /* tell user what one can do now                   */

exit


/* create the install script: this will copy the BSF4Rexx jar-file and dynamic link library
   to the appropriate Java extension directory
*/
createInstallScript: procedure expose j. cmd. path.
  fn=cmd.eInstallScriptName            /* name of install script              */
  ui=cmd.eUninstallScriptName          /* name of uninstall script            */

  call stream fn, "c", "open replace"
  call stream ui, "c", "open replace"

  hintString="created:" q(date("S") time()) "by user" q(j.user.name)

  call lineout fn, cmd.eLeadIn
  call lineout fn, cmd.eEcho qe(hintString)
  call lineout fn, cmd.eEcho qe("installing optional Java extension ""BSF4Rexx""...")
  call lineout fn, ""
  if cmd.eIsUnix=1 then                /* make sure rights are not fully available to the world */
  do
     call lineout fn, cmd.eLineComment "make sure rights are not fully available to the world"
     call lineout fn, "chmod 755 *.sh *.so *.CLS *.rex *.dll *.class *.REX"   /* set rwxr-xr-x */
     call lineout fn, "chmod 644 *.java"                                      /* set rw-r--r-- */
     call lineout fn, ""
  end

     /* uninstall: */
  call lineout ui, cmd.eLeadIn
  call lineout ui, cmd.eEcho qe("created:" q(date("S") time()) "by user" q(j.user.name))
  call lineout ui, cmd.eEcho qe("uninstalling optional Java extension ""BSF4Rexx""...")
  call lineout ui, ""


  tgtDir=j.java.ext.dirs
  call lineout fn, cmd.eEcho qe("copying the BSF4Rexx jar file(s) to the Java extension directory" q(tgtDir) "...")
  call lineout ui, cmd.eEcho qe("deleting the BSF4Rexx jar file(s) from the Java extension directory" q(tgtDir) "...")

  /* call lineout fn, cmd.eCopy qw(cmd.eJar) qw(tgtDir) */
  /* call lineout ui, cmd.eDelete qw(tgtDir || j.file.separator || cmd.eJarBaseName) */
  do i=1 to cmd.eJarBaseNames.0    /* iterate over all found jar-files    */
     call lineout fn, cmd.eCopy   qw(cmd.eJarBaseNames.i) qw(tgtDir)
     call lineout ui, cmd.edelete qw(tgtDir || j.file.separator || filespec("Name", cmd.eJarBaseNames.i) )
  end

  call lineout fn, ""
  call lineout ui, ""

  /* do we need to supply the commons.logging jar ?   */
  if j.rgf.commons.logging.available=0 then
  do
     call lineout fn, cmd.eEcho qe("/// NEED for org.apache.commons.logging ...")
/*
     call lineout fn, cmd.eEcho qe("NEED for org.apache.commons.logging, hence copying the file ...")
     shortName=filespec("Name", cmd.eCommonsLoggingJarName)

     call lineout fn, cmd.eEcho qe("copying the commons-logging jar" q(shortName)  "to the Java extension directory" q(tgtDir) "...")
     call lineout ui, cmd.eEcho qe("deleting the commons-logging jar" q(shortName) "from the Java extension directory" q(tgtDir) "...")
*/

     if cmd.eCommonsLoggingJarName="" then   /* hmm, not found in current directory!   */
     do
        msg="/// COULD NOT FIND the jar-file for the package org.apache.commons.logging AT INSTALL TIME !"
        call lineout fn, cmd.eEcho qe(msg)
        call lineout fn, cmd.eEcho qe("///  BSF4Rexx might not run! If so, then")
        call lineout fn, cmd.eEcho qe("///  look for the logging jar-file at http://jakarta.apache.org/commons")
        call lineout fn, cmd.eEcho qe("///  and copy the jar-file to" qw(tgtDir) "yourself")
        call lineout ui, cmd.eEcho qe(msg)
     end
/*
     else   /* create the copy/delete statement */
     do
        call lineout fn, cmd.eCopy   qw(cmd.eCommonsLoggingJarName) qw(tgtDir)
        call lineout ui, cmd.edelete qw(tgtDir || j.file.separator || filespec("Name", cmd.eCommonsLoggingJarName) )
     end
*/
     call lineout fn, ""
     call lineout ui, ""
  end



  tgtDir=cmd.eExt.dir.bin
  call lineout fn, cmd.eEcho qe("copying the BSF4Rexx dynamic link library file to its Java extension directory" q(tgtDir) "...")
  call lineout fn, cmd.eCopy qw(cmd.eDLL) qw(tgtDir)
  call lineout fn, ""
     /* uninstall: */
  call lineout ui, cmd.eEcho qe("delete the BSF4Rexx dynamic link library file from its Java extension directory" q(tgtDir) "...")
  call lineout ui, cmd.eDelete qw(tgtDir || j.file.separator || cmd.eDLL)
  call lineout ui, ""


  tgtDir=cmd.eLibDir
  call lineout fn, cmd.eEcho qe("copying the BSF4Rexx dynamic link library file to the lib directory" q(tgtDir) "...")
  call lineout fn, cmd.eCopy qw(cmd.eDLL) qw(tgtDir)
  call lineout fn, ""
     /* uninstall: */
  call lineout ui, cmd.eEcho qe("delete the BSF4Rexx dynamic link library file from the lib directory" q(tgtDir) "...")
  call lineout ui, cmd.eDelete qw(tgtDir || j.file.separator || cmd.eDLL)
  call lineout ui, ""


  tgtDir=cmd.eBinDir
  call lineout fn, cmd.eEcho qe("copying the Rexx scripts and ooRexx class files to" q(tgtDir) "...")
     /* uninstall: */
  call lineout ui, cmd.eEcho qe("deleting the Rexx scripts and ooRexx class files from" q(tgtDir) "...")
  list="*.rex *.REX *.cls *.CLS *.sh *.cmd"     /* list of files to copy/uninstall  */
  processed.=""
  do i=1 to words(list)
     call sysFileTree word(list, i), "files.", "FO"    /* get fully qualified files only  */
     do k=1 to files.0           /* now copy/delete file-by-file     */
        if cmd.eIsUnix=0 then    /* if Windows, then case-insensitive file-names, do not process them twice */
        do
           w=translate(files.k)  /* turn into uppercase              */
           if processed.w<>"" then iterate   /* already processed    */
           processed.w=files.k   /* remember that we processed this  */
        end

        call lineout fn, cmd.eCopy   qw(files.k) qw(tgtDir)   /* install    */
        call lineout ui, cmd.eDelete qw(makeFileName(tgtDir, j.file.separator, files.k))  /* uninstall  */
     end
  end


/* 2006-08-01: already handled in the above loop!
   /* copying setEnvironment4* to a PATH-directory */
  tgtDir=cmd.eBinDir
  -- tgtFile=stream(cmd.eSetEnvironmentScriptName, "C","Query EXISTS")
  tgtFile=cmd.eSetEnvironmentScriptName
  call lineout fn, cmd.eEcho qe("copying"  q(tgtFile) "to" q(tgtDir) "...")
  call lineout fn, cmd.eCopy qw(tgtFile) qw(tgtDir)
  call lineout fn, ""

  call lineout ui, cmd.eEcho qe("deleting" q(tgtFile)  "from directory" q(tgtDir) "...")
  call lineout ui, cmd.eDelete qw(tgtDir || j.file.separator || tgtFile)
  call lineout ui, ""
*/



/*
  if cmd.eIsUnix=1 then          /* Unix only, append ". setEnvironment..." to .bashrc  */
  do
     call lineout fn, ""
     call lineout fn, cmd.eEcho qc("Adding entries to the end of" cmd.eAdd2BashRc "...")

     appendTo=">>" cmd.eAdd2BashRc

     call lineout fn, cmd.eEcho qc(" ") appendTo
     call lineout fn, cmd.eEcho qc("#" hintString":") appendTo

     do i=1 to cmd.eAdd2BashRc.0
        call lineout fn, cmd.eEcho qc(cmd.eAdd2BashRc.i) appendTo
     end
     call lineout fn, ""
  end
*/


  call lineout fn, ""
  call lineout fn, cmd.eEcho qe("done.")
  call lineout fn, ""
  call lineout fn, cmd.eEcho cmd.eSupportInfos
  call lineout fn, ""

  call lineout ui, ""
  call lineout ui, cmd.eEcho qe("done.")
  call lineout ui, ""
  call lineout ui, cmd.eEcho cmd.eSupportInfos
  call lineout ui, ""

  call stream fn, "c", "close"
  call stream ui, "c", "close"

  if cmd.eSetExecutable<>"" then       /* set file executable (Unix)    */
  do
     cmd.eSetExecutable cmd.eInstallScriptName
     cmd.eSetExecutable cmd.eUninstallScriptName
  end
  return



/* create the test script which sets up all needed parameters to run Rexx programs from
   this directory */
createTestScript: procedure expose j. cmd. path.
  fn=cmd.eTestScriptName

  call stream fn, "c", "open replace"
  call lineout fn, cmd.eEchoOff
  call lineout fn, cmd.eLeadIn
  call lineout fn, cmd.eSetLocal       /* on Windows: create a local environment */
  call lineout fn, ""
  call lineout fn, cmd.eLineComment "created:" q(date("S") time()) "by user" q(j.user.name)
  call lineout fn, ""
  call lineout fn, cmd.eLineComment "script to run ""BSF4Rexx"" without installing it as a Java extension..."
  call lineout fn, ""

  call lineout fn, cmd.eLineComment "set CLASSPATH environment variable"
  call lineout fn, cmd.eSetCP          /* set CLASSPATH        */
  call lineout fn, ""

  call lineout fn, cmd.eLineComment "set PATH environment variable"
  call lineout fn, cmd.eSetPath        /* set PATH             */
  call lineout fn, ""

  call lineout fn, cmd.eLineComment "Java start command, allows passing arguments to Rexx script"
  call lineout fn, cmd.eTestStartRexx  /* Java start command   */
  call lineout fn, ""
  call lineout fn, cmd.eEcho qe("done.")
  call lineout fn, ""
  call lineout fn, cmd.eEcho cmd.eSupportInfos
  call lineout fn, ""

  call lineout fn, cmd.eEcho qe("usage:" fn "rexx-file-name optional_arguments")

  call stream fn, "c", "close"

  if cmd.eSetExecutable<>"" then       /* set file executable (Unix)    */
     cmd.eSetExecutable cmd.eTestScriptName

  return



/* creates the script that sets up the environment for Rexx to find the Java-DLL for
   demand loading it. */
createEnvironmentScript: procedure expose j. cmd. path.
  fn=cmd.eSetEnvironmentScriptName

  call stream fn, "c", "open replace"
  call lineout fn, cmd.eLeadIn
  call lineout fn, ""
  call lineout fn, cmd.eLineComment "created:" q(date("S") time())  "by user" q(j.user.name)
  call lineout fn, ""

  if cmd.eIsUnix=1 then
  do
     call lineout fn, cmd.eLineComment "run it once, after the optional Java extension ""BSF4Rexx"" got installed."
     call lineout fn, ""
     call lineout fn, "if ! [ -f /usr/lib/libjvm.so ] "
     call lineout fn, "then"
     call lineout fn, "   ln -sf" cmd.ePath2JavaDLL || "/libjvm.so"  "/usr/lib/libjvm.so"
     call lineout fn, "  " cmd.eEcho qe( q(cmd.ePath2JavaDLL || "/libjvm.so") "linked symbolically to" q("/usr/lib/libjvm.so") )
     call lineout fn, "else "
     call lineout fn, "  " cmd.eEcho qe( q("libjvm.so") "already available in" q("/usr/lib")", creation of a symbolic link not carried out!" )
     call lineout fn, "  " cmd.eEcho qe("to force new links to a specific Java version use:" q("rexx setupJava.rex"))
     call lineout fn, "fi   "

     call lineout fn, cmd.eEcho qe(" ")
     call lineout fn, cmd.eEcho qe("For non-Java programs to be able to successfully invoke Java via the")
     call lineout fn, cmd.eEcho qe( q('libvjm.so') "dynamic link library the following paths to Java directories")
     call lineout fn, cmd.eEcho qe("must be set (Java property" q('java.library.path') "on this system to:")
     call lineout fn, cmd.eEcho qe("   " pp(j.java.library.path))
     call lineout fn, cmd.eEcho qe(" ")

     call lineout fn, cmd.eEcho qe("Use the following command on the commandline:")
     call lineout fn, cmd.eEcho qe(" ")
     a="export LD_LIBRARY_PATH="j.java.library.path
     call lineout fn, cmd.eEcho qe(a)
     call lineout fn, a
     call lineout fn, cmd.eEcho qe(" ")
     call lineout fn, cmd.eEcho qe("or run this script like" q(". ./"cmd.eSetEnvironmentScriptName) -
                                   "to keep the changes to the environment.")
     call lineout fn, cmd.eEcho qe(" ")
     call lineout fn, cmd.eEcho qe("It is advised to add this environment setting to the profile of your system,")
     call lineout fn, cmd.eEcho qe("e.g. add the following entries to your" q('~/.bashrc')":")
     call lineout fn, cmd.eEcho qe(" ")
     call lineout fn, cmd.eEcho qe("   # allow ooRexx and other programs to find Java ("q('libjvm.so')")")
     call lineout fn, cmd.eEcho qe("   ." cmd.eSetEnvironmentScriptName ">/dev/null")
     call lineout fn, cmd.eEcho qe(" ")
  end
  else      /* non-Unix */
  do
     call lineout fn, cmd.eLineComment "just run it, before invoking a Rexx program that loads Java on its own."
     call lineout fn, ""

     call lineout fn, cmd.eEcho qe("setting the environment variable to point to the directory in which")
     call lineout fn, cmd.eEcho qe("""libjava.so"" (Linux) / ""java.dll"" (Windows) resides. This allows")
     call lineout fn, cmd.eEcho qe("Rexx programs to load Java on their own, if needed.")
     call lineout fn, ""

     call lineout fn, cmd.eSetDLLPath
     call lineout fn, ""

     call lineout fn, cmd.eEcho qe(".")
     call lineout fn, cmd.eEcho qe("It is advised to add this environment setting to your system's profile!")

     call lineout fn, cmd.eEcho qe("(This way it is always assured, that Rexx can find Java.)")
  end

  call lineout fn, ""
  call lineout fn, cmd.eEcho qe(".")
  call lineout fn, cmd.eEcho cmd.eSupportInfos
  call lineout fn, ""

  call stream fn, "c", "close"

  if cmd.eSetExecutable<>"" then       /* set file executable (Unix)    */
     cmd.eSetExecutable cmd.eSetEnvironmentScriptName

  return


/* create stem variable "sort." from "j." and sort it by property keys ascendingly */
dumpJavaProps: procedure expose j. cmd. path. sort.
   sort.=""
   max=0
   do i=1 to j.0
      sort.i=j.i.eName j.i.eValue
      max=max(max, length(j.i.eName))
   end
   max=max+2                     /* account for square brackets   */

   sort.0=j.0                    /* save total items in stem      */

   call sysStemSort "sort."      /* sort the stem                 */
   do i=1 to sort.0
      parse var sort.i n1 v1
      say right(i, 2) "-" left(pp(n1), max)":" pp(v1)
   end
   return


/* execute the Java program "JavaInfo4BSF4RexxInstallation.class" to queue all Java properties to
   the system Rexx queue and make them available via the stem variable "j."
*/
getJavaProps: procedure expose j. cmd. path.
   signal on any name usage                           /* trap to usage label     */
         /* get the Java properties    */
   if cmd.eJavaBinary="" then java="java"             /* default     */
                         else java=cmd.eJavaBinary    /* use command-line value  */

   java "-cp . JavaInfo4BSF4RexxInstallation | rxqueue"

   i=0
   max=0
   j.=""
   sort.=""
   if queued()=0 then
   do
      say "PANIC: nothing in external Rexx queue, cannot get at the Java properties, aborting..."
      say
      call usage
   end
   else
   do i=1 to 99999 while queued()<>0
      parse pull pName "=[" pVal "]"
      if pname="" then iterate      /* blank line (the last line?)   */
      max=max(max, length(pName))   /* memorize longest name         */

      parse var pVal v1 "0x" v2     /* hex-encoded string ?          */
      j.i.eName=pName               /* use a number based index      */
      j.i.eValue=v1||x2c(v2)

      j.pName=j.i.eValue            /* save in mixed case            */

      uName=translate(pName)        /* save in mixed case            */
      j.uName=j.i.eValue
      j.0=i                         /* save maximum number of entries*/
   end
   return



/* Get the fully qualified path to "jvm.dll", if multiple places, then
   prefer "client", "hotspot", "classic" (seems to be the only one), "server".
*/
getPath2JavaDll: procedure expose j. cmd. path.
   tmpStr=j.java.home

   parts="client hotspot classic server"  /* look in these directories  */

      /* removed search for [lib]javai.{so|dll}, as Java 1.1.8 is not supported anymore */
   if cmd.eIsUnix=1 then            /* Unix           */
      dlls="libjvm.so"
   else                             /* Windows, OS2   */
      dlls ="jvm.dll"

   countWords=words(dlls)
   do i=1 to countWords
      tmpFileName=j.java.home || j.file.separator || word(dlls, i)
      say "Attempt #" i "of" countWords "searching file: ["word(dlls, i)"] in ["j.java.home || j.file.separator || -
          "] or in one of its subdirectories..."
      call sysFileTree tmpFileName, "files.", "FOS"

      if files.0>0 then             /* files found?   */
      do
         do k=1 to words(parts)
            p=j.file.separator || word(parts, k) || j.file.separator    /* get word       */
            do m=1 to files.0       /* iterate over files found   */
               if pos(p, files.m)>0 then
               do
                  return substr(files.m, 1, lastpos(j.file.separator, files.m)-1)
               end
            end
         end
      end
   end

      /* default, even if directory does not exist to hint the user where to look for  */
   return tmpStr || j.file.separator || "bin" || j.file.separator || "client"


/* Parse commandline arguments, if any.
*/
parseArgs: procedure expose j. cmd. path.
   parse arg tmpStr
   if tmpStr="" then return ""      /* no arguments available        */
   tmpStr=strip(tmpStr)             /* remove leading & trailing ws  */
   if left(tmpStr,1)='"' then       /* argument enquoted             */
   do
      parse var tmpStr '"' path2javaExe '"' tmpStr
      path2javaExe='"' || path2javaExe || '"'
   end
   else
     parse var tmpStr path2javaExe tmpStr

   path2binDir=strip(strip(tmpStr), 'B', '"')   /* could be quoted on Windows (but not "apostrophed"!)   */
   binDir=path2binDir

   curDir=directory( )              /* save current directory        */
   newDir=directory( binDir )       /* See if directory exists       */
   if newDir \== binDir then
     signal usage
   z = directory( curDir )          /* restore current directory     */

   return path2javaExe || cmd.eArgSeparator || path2binDir


/*
   parse arg tmpStr
   if tmpStr="" then return ""      /* no arguments available        */
   tmpStr=strip(tmpStr)             /* remove leading & trailing ws  */
   if left(tmpStr,1)='"' then       /* argument enquoted             */
   do
      parse var tmpStr '"' path2javaExe '"' tmpStr
      path2javaExe='"' || path2javaExe || '"'
   end
   else
     parse var tmpStr path2javaExe tmpStr

   path2binDir=strip(tmpStr)

   return path2javaExe || cmd.eArgSeparator || path2binDir
*/



/* Get path to Rexx or Regina from PATH environment variable and return it;
   Windows/OS2
*/
getRexxDir: procedure expose j. cmd. path.
     /* use this precedence order to locate a path where to copy Rexx-files to  */
  dir=findPath("\ooREXX \regina rexx")
  if dir<>"" then return dir

  eyeCatcher="?!no-rexx-path-found!?"
  say "PROBLEM: could not find explicit path to a Rexx-directory!"
  say "         Please adjust the created scripts to point to any one directory listed"
  say "         with the environment variable ""PATH"", ie. replace" pp(eyecatcher) "with"
  say "         any of the listed PATH directory, and then run the scripts."
  return eyecatcher              /* indicate that not found       */


/* Look for filename in path, if found return fully qualified path.
*/
getFromPath: procedure expose j. cmd. path.
  parse arg filename

  do i=0 to path.0      /* iterate over path directories          */
     fn=stream(path.i || j.file.separator || filename, "C", "QUERY EXISTS")

     if fn<>"" then return fn /* found, return it                 */
  end
  return filename



/* Extract filename from fully qualified path in "file" and add it to "tgtDir".
*/
makeFileName: procedure
  parse arg tgtDir, sep, file
  lpos=lastpos(sep, file)
  if lpos>0 then file=substr(file, lpos+1)
  return tgtDir || sep || file

pp: procedure                    /* poor man's "pretty" print     */
  return "[" || arg(1) || "]"


q: procedure                     /* enclose string into apostrophes  */
  return '"' || arg(1) || '"'


   /* "qe": quote echo string: enquote entire string on Unix only */
qe: procedure expose j. cmd. path.
  if  cmd.eIsUnix=1 then return "'" || arg(1) || "'"  /* use apostrophes only, rgf 2006-01-18   */
                    else return arg(1)


   /* "qe": quote fully-qualified paths in Windows as they may contain a blank */
qw: procedure expose j. cmd. path.
  if  cmd.eIsUnix=0 then return """" || arg(1) || """"
                    else return arg(1)

   /* "qc": quote string in apostrophes, if Unix, in quotes, if Windows (for support infos that contain <, >) */
qc: procedure expose j. cmd. path.
  if  cmd.eIsUnix=0 then return '"' || arg(1) || '"'     /* Windows  */
                    else return "'"  || arg(1) || "'"    /* Unix     */


iif: procedure                   /* VBasic-like function          */
  if arg(1)=1 then return arg(2)
  return arg(3)

info:                            /* tell user what has happened   */
  say
  say "the following scripts have been created:"
  say
  say " " pp(cmd.eTestScriptName)": script to run ""BSF4Rexx"" without installing it as a Java extension"
  say
  if cmd.eIsUnix=0 then
  do
     say "       usage:" cmd.eTestScriptName "rexx-file optional-arguments"
     say "     example:" cmd.eTestScriptName "infoBSF.rex"
  end
  else
  do
     say "       usage:" q("./"cmd.eTestScriptName "rexx-file optional-arguments")
     say "     example:" q("./"cmd.eTestScriptName "infoBSF.rex")
  end

  say
  say
  say " " pp(cmd.eInstallScriptName)": install" q("BSF4Rexx") "as an optional Java extension (recommended)"
  say
  if cmd.eIsUnix=0 then
     say "       usage:" cmd.eInstallScriptName
  else
     say "       usage:" q("./"cmd.eInstallScriptName)

  say
  say "     remarks: - You may need to run as user ""root"" (Linux) or ""Administrator"" (Windows)."
  say "              - After installation you may start any Rexx-program by issuing:"
  say
  if cmd.eIsUnix=0 then
     say "                  " cmd.eSetEnvironmentScriptName

  say "                   rexx rexx-file optional-arguments"
  say
  say "                or (note the trailing ""j"", which means via ""Java""):"
  say

  if cmd.eIsUnix=0 then
     say "                   rexxj rexx-file optional-arguments"
  else
     say "                   rexxj.sh rexx-file optional-arguments"

  say
  say " " pp(cmd.eSetEnvironmentScriptName)": allows Rexx to find Java ("q("jvm.dll")") on its own"

  if cmd.eIsUnix=1 then
     say "                it is advised to run this script right after installation of ""BSF4Rexx"""
  else
     say "                it is advised to add this environment setting to your system's profile"
  say

  say " " pp(cmd.eUninstallScriptName)": use to uninstall the optional Java extension" q("BSF4Rexx")
  say
  if cmd.eIsUnix=0 then
     say "       usage:" cmd.eUninstallScriptName
  else
     say "       usage:" q(cmd.eUninstallScriptName)

  say
  say "     remarks: - You may need to run as user ""root"" (Linux) or ""Administrator"" (Windows)."
  say
  say "Good luck! :-)  [You may also want to take a look at these generated scripts.]"
  exit 0

/* give user feedback and show usage of this script */
usage:
  say
  say "This program creates scripts to test, install and uninstall ""BSF4Rexx""."
  say
  say "Usage:     rexx setupBSF [fully-qualified-file-name-of-java.exe [dir-to-copy-scripts-to]]"
  say "  (dir-to-copy-scripts-to MUST exist)"
  say
  say "Examples:  rexx setupBSF"

  if cmd.eIsUnix=0 then
  do
     say "           - will use /usr/bin/java"
     say "             if no java binary is found, then execute ""rexx setupJava.rex"", which"
     say "             will create the necessary symbolic links to an installed Java."
     say "           - will copy the BSF4Rexx-support-files to /usr/bin"
     say
  end
  else
  do
     say "           - will use any ""java.exe"" found in one of the directories of the"
     say "             environment variable ""PATH"""
     say "           - will copy the BSF4Rexx-support-files to the Rexx/Regina directory, "
     say "             if found in the environment variable ""PATH"""
  end

  say
  if cmd.eIsUnix=0 then
  do
     say "           rexx setupBSF E:\jdk1.5\jre\bin\java.exe D:\windows\system32"
     say "           - will use ""java.exe"" from the given first argument"
     say "           - will copy the BSF4Rexx-support-files to the Windows system directory"
  end
  else
  do
     say "           rexx setupBSF /usr/java/j2sdk1.4.2_10/jre/bin/java /usr/bin"
     say "           - will use the binary ""java"" from the given first argument"
     say "           - will copy the BSF4Rexx-support-files to /usr/bin"
  end

  exit -1


/* parse path into stem path. */
parsePath: procedure expose j. cmd. path.
/*
say "... j.path.separator=["j.path.separator"]"
say "... j.file.separator=["j.file.separator"]"
*/

  p=value("PATH", , "ENVIRONMENT")
  i=0
  do while p<>""
     parse var p chunk (j.path.separator) p
     i=i+1

     if cmd.eIsUnix=1 then
        path.i=chunk
     else
        path.i=changestr('"', chunk, "")  /* remove any quotes from any part of the Windows path */

     path.i.upperCase=translate(chunk)  /* uppercase version */
/*
say "... i="i "chunk="pp(chunk)
*/

     path.0=i
  end

  return

/* find needle in path and return it   */
findPath: procedure expose j. cmd. path.
   parse upper arg needles    /* blank delimited, translated into uppercase   */
   do i=1 to words(needles)   /* iterate over needles                         */
      n=word(needles, i)
      do k=1 to path.0        /* iterate over paths      */
         if pos(word(needles,i), path.k.upperCase)>0 then
            return path.k
      end
   end
   return ""

